#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>     /* for _delay_terrupt.h> */
#include <avr/pgmspace.h>   /* required by usbdrv.h */


#include "peri.h"
#include "usbdrv.h"

#define RQ_GET_RED    0
#define RQ_GET_GREEN  1
#define RQ_GET_BLUE   2
#define RQ_GET_YELLOW 3
#define RQ_GET_BLACK  4
#define RQ_GET_SWITCH 5

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state; 

    if (rq->bRequest == RQ_GET_RED)
    {
        switch_state = ((PINC & (1<<PC2)) == 0);

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    else if (rq->bRequest == RQ_GET_GREEN)
    {
        switch_state = ((PINC & (1<<PC0)) == 0);

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    else if (rq->bRequest == RQ_GET_BLUE)
    {
        switch_state = ((PINC & (1<<PC1)) == 0);

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    
    else if (rq->bRequest == RQ_GET_YELLOW)
    {
        switch_state = ((PINC & (1<<PC3)) == 0);

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    else if (rq->bRequest == RQ_GET_BLACK)
    {
        switch_state = ((PINC & (1<<PC4)) == 0);

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    else if (rq->bRequest == RQ_GET_SWITCH)
    {
	switch_state = ((PINC & (1<<PC5)) == 0);
	usbMsgPtr = &switch_state;
	return 1;
    }

    else
    {
        /* default for not implemented requests: return no data back to host */
        return 0;
    }
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peri();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}



